//Project: 
//Date: Oct 31, 2006
//Author: andi
//File name: TextTerm.java

package com.jcraft.jcterm;

import com.jcraft.jsch.ChannelShell;

import javax.swing.*;
import java.awt.*;
import java.io.IOException;


/**
 * @author Andrei Diaconu-Muresan
 */
public class TextTerm extends JTextArea implements Term {

	public static final char BLANK = ' ';
	public static final char CURSOR = '=';
	private int cursorX;
	private int cursorY;
	private int rowCount;
	private int columnCount;
	private Object background;
	private Object foreground;
	private Emulator emulator;
	private ChannelShell shell;


	/**
	 * Constructs a new TextArea.  A default model is set, the initial string
	 * is null, and rows/columns are set to 0.
	 */
	public TextTerm(ChannelShell shell) throws IOException {
		this.shell = shell;
		this.setSize(40, 40);
		this.emulator = new EmulatorVT100(this, shell.getInputStream());
		new Thread() {

			@Override
			public void run() {
				TextTerm.this.emulator.start();
			}
		}.start();
	}

	private char[][] charMatrix;

	public int getRowCount() {
		return rowCount;
	}

	public int getColumnCount() {
		return columnCount;
	}

	public int getCharWidth() {
		return 5;
	}

	public int getCharHeight() {
		return 5;
	}

	public void setCursor(int x, int y) {
		cursorX = x;
		cursorY = y;
	}

	public void clear() {
		for (int i = 0; i < rowCount; i++) {
			for (int j = 0; j < columnCount; j++) {
				charMatrix[i][j] = BLANK;
			}
		}
		this.sync();
	}

	public void draw_cursor() {
		charMatrix[cursorX][cursorY] = CURSOR;
		this.sync();
	}

	public void redraw(int x, int y, int width, int height) {
	}//void redraw();

	public void clear_area(int x1, int y1, int x2, int y2) {
		for (int i = x1; i < x2; i++) {
			for (int j = y1; j < y2; j++) {
				charMatrix[i][j] = BLANK;
			}
		}
		this.sync();
	}

	public void scroll_area(int x, int y, int w, int h, int dx, int dy) {


	}

	public void drawBytes(byte[] buf, int s, int len, int x, int y) {
		for (int i = s; i < s + len; i++) {
			charMatrix[x + i][y] = (char) buf[i];

		}
		this.sync();
	}

	public void drawString(String str, int x, int y) {
		this.drawBytes(str.getBytes(), 0, str.length(), x, y);
		this.sync();
	}

	public void beep() {
		Toolkit.getDefaultToolkit().beep();
	}

	public void setFGround(Object foreground) {
		this.foreground = foreground;
	}

	public void setBGround(Object background) {
		this.background = background;
	}

	public Object getFGround() {
		return foreground;
	}

	public Object getBGround() {
		return background;
	}

	private void sync() {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < charMatrix.length; i++) {
			char[] chars = charMatrix[i];
			sb.append(chars).append('\n');
		}
		super.setText(sb.toString());
	}

	public void setSize(int width, int height) {
		this.rowCount = width;
		this.columnCount = height;
		super.setColumns(columnCount);
		super.setRows(rowCount);
		this.charMatrix = new char[width][height];
		for (char[] chars : charMatrix) {
			for (int i = 0; i < chars.length; i++) {
				chars[i] = ' ';

			}
		}
		if (this.emulator != null)
			emulator.reset();
	}
}
