//Project: 
//Date: Oct 30, 2006
//Author: andi
//File name: SSHAction.java

package org.itbikers.idea.ssh.actions;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.project.Project;
import org.itbikers.idea.ssh.client.ConnectionData;
import org.itbikers.idea.ssh.ui.runonce.SSHConfigurationDialog;
import org.itbikers.idea.ssh.ui.toolwindow.SSHToolWindow;

import javax.swing.*;


/**
 * Action that is invoked in order to create a new SSH terminal.
 *
 * @author Andrei Diaconu-Muresan
 */
public class SSHAction extends AnAction {

	/**
	 * If there exists a tool window with ssh terminals, a new terminal is created,
	 * otherwise a new terminal is added to the ssh tool window.
	 *
	 * @param e the event object.
	 */
	public void actionPerformed(AnActionEvent e) {
		final Project project = (Project) e.getDataContext().getData(DataConstants.PROJECT);
		createConnection(project);
	}

	/**
	 * Creates a new connection.
	 *
	 * @param project the current project.
	 */
	private void createConnection(Project project) {
		final ConnectionData conectionData = getConectionData();
		if (conectionData == null)
			//user canceled connection
			return;
		final SSHToolWindow toolWindow = SSHToolWindow.getInstance(project);
		toolWindow.createTerminal(conectionData);
	}

	/**
	 * Gets the data from the user.
	 *
	 * @return the data from the user or null if the user has canceled the connection.
	 */
	private ConnectionData getConectionData() {
		final SSHConfigurationDialog dialog = new SSHConfigurationDialog();
		dialog.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		dialog.pack();
		dialog.setLocationRelativeTo(null);
		dialog.setModal(true);
		dialog.setVisible(true);
		if (dialog.isOk())
			return dialog.getConnectionData();
		else
			return null;
	}
}
