package org.itbikers.idea.ssh.ui.runonce;

import com.intellij.ui.DocumentAdapter;
import org.itbikers.idea.ssh.client.ConnectionData;
import org.itbikers.idea.ssh.resources.messages.SSHBundle;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import java.awt.event.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class SSHConfigurationDialog extends JDialog {
	private JPanel contentPane;
	private JButton buttonOK;
	private JButton buttonCancel;
	private JTextField connectToTextField;
	private boolean ok;

	/**
	 * Pattern for the connection string.
	 * <br/>
	 * Readable form:<br/>
	 * user[:password]@host[:port]
	 */
	private Pattern pattern = Pattern.compile("([^:]+)(:([^@:]+))?@([^:@]+)(:(\\d+))?");
	private ConnectionData connectionData;

	public SSHConfigurationDialog() {
		setContentPane(contentPane);
		setTitle(SSHBundle.message("ssh.runonce.connectto.title"));
		setModal(true);
		getRootPane().setDefaultButton(buttonOK);

		buttonOK.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				onOK();
			}
		});

		buttonCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				onCancel();
			}
		});

// call onCancel() when cross is clicked
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				onCancel();
			}
		});

// call onCancel() on ESCAPE
		contentPane.registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				onCancel();
			}
		}, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		init();
	}

	private void init() {
		this.connectToTextField.getDocument().addDocumentListener(new DocumentAdapter() {
			protected void textChanged(DocumentEvent e) {
				SSHConfigurationDialog.this.validateField();
			}
		});
		this.connectionData = new ConnectionData();
		this.validate();
	}


	private void onOK() {
		this.ok = true;
		dispose();
	}

	private void onCancel() {
		this.ok = false;
		dispose();
	}

	/**
	 * Validates the field against the pattern.
	 */
	private void validateField() {
		final Matcher matcher = pattern.matcher(connectToTextField.getText());
		if (matcher.matches()) {
			this.buttonOK.setEnabled(true);
			this.connectionData.setUsername(matcher.group(1));
			this.connectionData.setPassword(matcher.group(3));
			this.connectionData.setHostname(matcher.group(4));
			final String port = matcher.group(6);
			if (port != null)
				this.connectionData.setPort(Integer.parseInt(port));
			else
				this.connectionData.setPort(null);
		} else
			this.buttonOK.setEnabled(false);
	}


	public ConnectionData getConnectionData() {
		return connectionData;
	}

	/**
	 * Returns true if the dialoged was closed by hitting the OK button, or by pressing
	 * <em>enter</em>.
	 *
	 * @return true if user accepted the form.
	 */
	public boolean isOk() {
		return ok;
	}
}
