//Project: 
//Date: Oct 30, 2006
//Author: andi
//File name: TerminalTab.java

package org.itbikers.idea.ssh.ui.toolwindow;

import com.jcraft.jcterm.JCTerm;
import org.itbikers.idea.ssh.client.ConnectionData;
import org.itbikers.idea.ssh.resources.messages.SSHBundle;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;


/**
 * Component to be placed in a tab in {@link SSHToolWindow}.
 *
 * @author Andrei Diaconu-Muresan
 */
public class TerminalTab extends JPanel implements FocusListener {
	/**
	 * True if the connection has been closed.
	 */
	private boolean ended;
	/**
	 * The terminal.
	 */
	private JCTerm jcTerm;
	private JScrollPane scrollPane;
	private Border focusBorder;
	private Border noFocusBorder;


	/**
	 * Creates a new terminal tab.
	 *
	 * @param connectionData the connection data.
	 */
	public TerminalTab(final ConnectionData connectionData) {
		super(new BorderLayout());
		this.setFocusable(true);
		this.focusBorder = BorderFactory.createBevelBorder(BevelBorder.RAISED);
		this.noFocusBorder = BorderFactory.createBevelBorder(BevelBorder.LOWERED);
		jcTerm = new JCTerm();
		jcTerm.setTerminalTab(this);
		jcTerm.setUserHost(connectionData.getUsername() + '@' + connectionData.getHostname());
		jcTerm.setPort(connectionData.getPort());
		jcTerm.setPassword(connectionData.getPassword());
		jcTerm.setCommand(connectionData.getCommand());
		scrollPane = new JScrollPane(jcTerm);
		scrollPane.setFocusable(true);
		this.add(scrollPane, BorderLayout.CENTER);
		jcTerm.kick();
	}

	/**
	 * Closes the terminal. The connection is also closed.
	 */
	public void close() {
		jcTerm.close();
	}

	public void connectionClosed() {
		if (!ended)
			this.ended = true;
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				TerminalTab.this.replacePanel();
			}
		});
	}

	/**
	 * Removes the terminal panel and adds a label.
	 */
	private void replacePanel() {
		this.removeAll();
		final String text;
		Exception e = jcTerm.getException();
		if (e != null)
			text = SSHBundle.message("ssh.terminal.closed.label.error", e.getMessage());
		else
			text = SSHBundle.message("ssh.terminal.closed.label");
		this.add(new Label(text));
	}

	/**
	 * Sets the focus on tjis termina.
	 */
	public void setFocus() {
//		jcTerm.requestFocus();
	}

	/**
	 * Invoked when a component gains the keyboard focus.
	 */
	public void focusGained(FocusEvent e) {
		scrollPane.setBorder(focusBorder);
	}

	/**
	 * Invoked when a component loses the keyboard focus.
	 */
	public void focusLost(FocusEvent e) {
		scrollPane.setBorder(noFocusBorder);
	}
}
